;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "no-littering" "20260817.404"
  "Help keeping ~/.config/emacs clean."
  '((emacs  "28.1")
    (compat "31.0"))
  :url "https://github.com/emacscollective/no-littering"
  :commit "22235d2790d086db418c1b487e6cd700a8fa8237"
  :revdesc "22235d2790d0"
  :keywords '("convenience")
  :authors '(("Jonas Bernoulli" . "emacs.no-littering@jonas.bernoulli.dev"))
  :maintainers '(("Jonas Bernoulli" . "emacs.no-littering@jonas.bernoulli.dev")))
